/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#ifndef DECODERPOSTERIORVITERBI_H_
#define DECODERPOSTERIORVITERBI_H_
#include "Decoder.h"
#include "ForwardBackward.h"
#include "Model.h"

namespace decoding {

class DecoderPosteriorViterbi : public Decoder
{
protected:	
	algo::ForwardBackward * fb;
public:
	DecoderPosteriorViterbi(models::Model * model) : Decoder(model) {
		this->fb = new algo::ForwardBackward(1, this->model->transducer->num_states);
		this->fb->bos = model->options->include_bos_feature;
		this->fb->eos = model->options->include_eos_feature;
	}
	~DecoderPosteriorViterbi(){
		std::cout << "Destroying decoder posterior viterbi" << std::endl;
		if (fb != NULL)
			delete fb;
		if (vt != NULL)
			delete vt;
		if (lambda != NULL)
			delete lambda;
	}
	void predict(data::Sequence & seq);
	
};

}
#endif /*DECODERPOSTERIORVITERBI_H_*/

