/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#ifndef FORWARDBACKWARD_H_
#define FORWARDBACKWARD_H_
#include "DoubleMatrix.h"
#include "DoubleVector.h"
#include "Data.h"
#include "FeatureSet.h"

namespace models {
	class Model;
}

namespace algo {

class ForwardBackward
{
protected:
	int index;	
	
public:

	bool clamped;
	bool scaling;
	bool sum_on_labels;
	bool bos;
	bool eos;
	int seq_len;
	int num_states;
	
	int beg_idx, end_idx;
	
	double Zx;
	
	
	mathlib::DoubleVector * fw, * next_fw;
	mathlib::DoubleMatrix * bw_m;
	mathlib::DoubleMatrix * Mi;
	mathlib::DoubleVector * Vi;
	double * scale;	
	
	ForwardBackward(int seq_len, int num_states);
	~ForwardBackward();
	void reset(int seq_len);
	void run(data::Sequence & seq, models::Model * model, double * weights);
	void run_backward(data::Sequence & seq, models::Model * model, double * weights);
	void start_forward_scan();
	void take_forward_step(data::Sequence & seq, models::Model * model, double * weights);
	double current_forward(int state);
	double current_backward(int state);
	double p(int state);
	double p(int from_state, int to_state);
	double p_i(int state, int pos);
	double p_i(int from_state, int to_state, int pos);
	double get_partition_function();
	double compute_scaling_adjustment();
	void set_beg_idx(int beg_idx);
	void set_end_idx(int end_idx);

};

}
#endif /*FORWARDBACKWARD_H_*/

