/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#ifndef TRANSDUCER_H_
#define TRANSDUCER_H_

#include <string>
#include <map>
#include <fstream>
#include "DoubleMatrix.h"
#include "Options.h"
#include "Data.h"
#include "ForwardBackward.h"

namespace models {
	class Model;
}

namespace grammar {

typedef std::map<int,std::string> ist2str;

typedef std::map<std::string,int> sst2int;

typedef std::map<int,int> ist2lbl;

typedef std::multimap<int,int> ilb2sta; 

typedef std::map<int,std::string> ilb2str;

typedef std::map<std::string,int> slb2int;

class Transducer
{
public:
	
	models::Model * model;
	
	int num_labels;
	
	int num_states;
	
	mathlib::DoubleMatrix * transitions;
	mathlib::DoubleMatrix * labelsmap;
	mathlib::DoubleMatrix * bw;
	mathlib::DoubleMatrix * fw;
	
	ist2str * dictstint;
	
	sst2int * dictststr;
	
	ist2lbl * stlblmap;
	
	ilb2sta * lblstmmap;
	
	ilb2str * dictlbint;
	
	slb2int * dictlbstr;
	
	Transducer(std::ifstream * model_file);
	
	Transducer(std::string topology_file);
	
	~Transducer();
	
	void simulate(data::Sequence & seq);
	void simulate_ex(data::Sequence & seq);
	
	std::vector<int> get_states(int pos);
	
	void dump(std::ofstream * os);
private:
	void read_topology(std::ifstream * is);
};
}
#endif /*TRANSDUCER_H_*/

