/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#ifndef VITERBI_H_
#define VITERBI_H_
#include "Data.h"
#include "DoubleMatrix.h"
#include "DoubleVector.h"
#include "ForwardBackward.h"

namespace models {
	class Model;
}

namespace algo {

class Viterbi
{
public:

	mathlib::DoubleMatrix * memory;
	int **indexes;

	mathlib::DoubleMatrix * Mi;
	mathlib::DoubleVector * Vi;

	double * scale;
	
	int seq_len;
	int num_states;
	int beg_idx;
	int end_idx;
	
	bool scaling;
	bool bos;
	bool eos;
	
	Viterbi(int seq_len, int num_states);
	virtual ~Viterbi();
	void reset(int seq_len);
	void posterior_viterbi(data::Sequence & seq, ForwardBackward * fb, models::Model * model);
	void viterbi(data::Sequence & seq, models::Model * model, double * weights);
	void backtrace(data::Sequence & seq, models::Model * model);
};

}
#endif /*VITERBI_H_*/

