/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * Data.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <math.h>
#include "Data.h"
#include "Tokenizer.h"
#include "DoubleMatrix.h"
#include "DoubleVector.h"
#include "FeatureSet.h"
#include "Model.h"

namespace data {

void Observation::compute_state_feature_scores(features::FeatureSet * fs, std::vector<Observation> & seq, int pos, mathlib::DoubleVector * Vi, double * weights)
{
	int j;
	
	*Vi = 0.0;
	if (pos != seq.size()-1 && pos != 0) {
	for (j=0; j<features.size(); j++)
	{
	  if (fs->features[features[j]].type == STATE_FEATURE)
	  {
	    Vi->set(fs->features[features[j]].to_state, Vi->get(fs->features[features[j]].to_state)+weights[features[j]]*
	    		atof(seq[pos+fs->features[features[j]].window_offset].cp[fs->features[features[j]].cp].c_str()) );
	  }
	}
	/*	
	for (boost::dynamic_bitset<>::size_type i = 0; i < feat_mask.size(); ++i){
		if (feat_mask[i] && fs->features[i].type == STATE_FEATURE) {
			//Vi->set(fs->features[i].to_state, Vi->get(fs->features[i].to_state)+weights[i]*
				//atof(seq[pos+fs->features[i].window_offset].cp[fs->features[i].cp].c_str()));
			Vi->set(fs->features[i].to_state, Vi->get(fs->features[i].to_state)+weights[i]*
				(double)(seq[pos+fs->features[i].window_offset].cp[fs->features[i].cp]/100.0));
		}
	}
	*/
	}
	
	/*
	std::map<int,double>::iterator fit;
	
	
				
	for (fit = features.begin(); 
				fit != features.end(); ++fit) {
		int type = fs->features[fit->first].type;
		if (type == STATE_FEATURE) {
			double lambda = weights[fit->first];
			int to = fs->features[fit->first].to_state;
			
			Vi->set(to, Vi->get(to)+lambda*fit->second);
		}
	}
	*/
	for (j=0; j<Vi->size; j++) {
		Vi->set(j, exp(Vi->get(j)));
	}
}

void Observation::compute_transition_feature_scores(features::FeatureSet * fs, mathlib::DoubleMatrix * Mi, double * weights)
{
	
	int k, j;
	
	*Mi = 0.0;
	
	for (j=0; j<features.size(); j++)
	{
	  if (fs->features[features[j]].type == TRANS_FEATURE)
	  {
	    Mi->set(fs->features[features[j]].from_state, fs->features[features[j]].to_state,
	              Mi->get(fs->features[features[j]].from_state,fs->features[features[j]].to_state)+weights[features[j]]);

	  }
	}

	
	/*	
	for (boost::dynamic_bitset<>::size_type i = 0; i < feat_mask.size(); ++i){
		if (feat_mask[i] && fs->features[i].type == TRANS_FEATURE) {
			
			Mi->set(fs->features[i].from_state, fs->features[i].to_state, 
				Mi->get(fs->features[i].from_state,fs->features[i].to_state)+weights[i]);
		}
	}
	
	
	std::map<int,double>::iterator fit;
	
	
	for (fit = features.begin(); 
				fit != features.end(); ++fit) {
		int type = fs->features[fit->first].type;
		if (type == TRANS_FEATURE) {
			double lambda = weights[fit->first];
			int to = fs->features[fit->first].to_state;
			int from = fs->features[fit->first].from_state;
			Mi->set(from, to, Mi->get(from,to)+lambda*fit->second);
		}
	}
	*/
	for (k=0; k<Mi->cols; k++) {
		for (j=0; j<Mi->rows; j++) {	
			Mi->set(j, k, exp(Mi->get(j, k)));
		}
	}
	
}

Data::Data()
{
}

Data::~Data()
{
	if (ds != NULL)
		delete ds;
}

void Data::read(std::string sequences_file) 
{
	std::string line;
	char buffer[BUFF_SIZE_LONG];
	//std::filebuf fb;
	//fb.open(sequences_file.c_str(), std::ios::in);
	//std::istream is(&fb);
	//
	
	std::ifstream is(sequences_file.c_str());
        if (! is.is_open()) {
         std::cout << "Error opening/reading sequences file" << std::endl;
         exit(EXIT_FAILURE);
        }
	Sequence seq;
	
	ds = new Dataset;
	
	Observation obs;
	obs.label = 0;
	obs.prediction = -1;
	obs.cp.push_back("0.00");
	//obs.cp.push_back(0);
	seq.push_back(obs);
	
	
	while (! is.getline(buffer, BUFF_SIZE_LONG).eof()) {
		line = buffer;
		utils::Tokenizer tok(line, " \t\r\n");
		
		int len = tok.count_tokens();
		
		if (len <= 0) {
			
			Observation ob;
			ob.label = 1;
			ob.prediction = -1;
			ob.cp.push_back("0.00");
			//ob.cp.push_back(0);
			seq.push_back(ob);
			ds->push_back(seq);
			seq.clear();
			
			ob.label = 0;
			ob.prediction = -1;
			ob.cp.push_back("0.00");
			//ob.cp.push_back(0);
			seq.push_back(ob);
		} else {
			Observation ob;
			
			int endpos;
			if (model->options->ispredicting){
			  endpos = len;
			  ob.label = -1;
			} else {
  			  std::map<std::string,int>::iterator lmap;
              lmap = this->model->transducer->dictlbstr->find(tok.token(len-1));
			  if (lmap == this->model->transducer->dictlbstr->end()){
			    std::cout << "Label " << tok.token(len-1) << " not found in sequence " << ds->size()+1 << " at position " << seq.size()+1 << "." << std::endl;
			    exit(EXIT_FAILURE);
			  }
                          ob.label = lmap->second;//atoi(tok.token(len-1).c_str());
			  endpos = len-1;
			}

			ob.prediction = -1;
			//std::cout << (double)((short)atoi(tok.token(5).substr(2,2).c_str()))/100.0 << std::endl;
			for (int i = 0; i<endpos; i++){
				//if(tok.token(i).compare("1.00") != 0){
				//ob.cp.push_back((short)atoi(tok.token(i).substr(2,2).c_str()));
				//} else {
				//ob.cp.push_back(100);
				//}
				ob.cp.push_back(tok.token(i));
			}
			
			seq.push_back(ob);
		}
	}
	num_sequences = ds->size();
	//std::cout << "Number of sequences: " << num_sequences << std::endl;
	//is.close();
}

void Data::dump() 
{
	int pos, seq;
	data::Dataset::iterator datait;
	std::map<int,double>::iterator featit;
	
	seq = 0;
	for (datait = ds->begin() ; 
			datait != ds->end(); datait++) {
		int seqlen = datait->size();
		std::cout << "Sequence " << seq << std::endl;
		for (pos = 1 ; pos < seqlen-1; pos++) {
			std::cout << "Position " << pos << std::endl;
			/*
			for (featit = (*datait)[pos].features.begin(); featit != (*datait)[pos].features.end() ; featit++) {
				std::cout << "Feature = (" << featit->first << ", " << featit->second << ")" << std::endl;
			}*/
		}
		std::cout << std::endl;
		seq++;
	}
}

void Data::write_predictions()
{
	int pos;
	
	data::Dataset::iterator datait;
	std::map<int,std::string>::iterator lbldictit;
	
	/*
	std::filebuf fb;
	fb.open(this->model->options->prediction_file.c_str(), std::ios::out);
	std::ostream os(&fb);
	*/
	std::ofstream os(this->model->options->prediction_file.c_str(), std::ios_base::trunc);
	if (! os.is_open()) {
		std::cout << "Error opening output file" << std::endl;
		exit(EXIT_FAILURE);			   
	}
	for (datait = ds->begin() ; 
		datait != ds->end(); datait++) {
		int seqlen = datait->size();
		for (pos = 1 ; pos < seqlen-1; pos++) {
			if (! this->model->options->ispredicting){
			lbldictit = this->model->transducer->dictlbint->find((*datait)[pos].label);
			os << lbldictit->second << "\t";
			}
			lbldictit = this->model->transducer->dictlbint->find((*datait)[pos].prediction);
			os << lbldictit->second << std::endl;
		}
		os << std::endl;
	}
	os.close();
	
}
}


