/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * DecoderFactory.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include "DecoderFactory.h"
#include "DecoderViterbi.h"
#include "DecoderPosteriorViterbi.h"
#include "DecoderPosteriorViterbiMax.h"
#include "DecoderPosteriorViterbiSum.h"
#include "Options.h"
#include "Model.h"

namespace decoding {

DecoderFactory::DecoderFactory()
{
}

DecoderFactory::~DecoderFactory()
{
}

Decoder * DecoderFactory::get_decoder(int decoder_type, models::Model * model) 
{
	Decoder * res;
	
	switch (decoder_type) {
	
		case VITERBI:
			res = new DecoderViterbi(model);
			break;
		case POSTERIORMAX:
			res = new DecoderPosteriorViterbiMax(model);
			break;
		case POSTERIORSUM:
			res = new DecoderPosteriorViterbiSum(model);
			break;
		default:
			res = new DecoderViterbi(model);
			break;
	}
	
	return res;
}

}
