/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * DoubleVector.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include <stdio.h>
#include <iostream>
#include "DoubleVector.h"
#include "MathlibException.h"

namespace mathlib
{

DoubleVector::DoubleVector(int size)
{
	try {
	if (size > 0) {
	this->size = size;
	//vect = gsl_vector_alloc(size);
	this->vect = new double[size];
	} else {
		throw MathlibNoValidSizeException("DoubleVector::DoubleVector");
	}
	} catch (MathlibException & e) {
		e.report();
	}
}
	
void DoubleVector::set(int i, double value)
{
	try {
	if (i >= 0 && i < this->size) {
	//gsl_vector_set(vect, i, value);
	vect[i] = value;
	} else {
		throw MathlibIndexOutOfRangeException("DoubleVector::set");
	}
	} catch (MathlibIndexOutOfRangeException & e) {
		e.report();
	}
	
}

double DoubleVector::get(int i)
{
	double res;
	try {
	if (i >= 0 && i < size) {
	//res = gsl_vector_get(vect, i);
	res = vect[i];
	} else {
		throw MathlibIndexOutOfRangeException("DoubleVector::get");
	}
	} catch (MathlibIndexOutOfRangeException & e) {
		e.report();
	}
	return res;
}

double DoubleVector::sum()
{
	int i;
	double res = 0.0;
	for (i=0; i<size; i++)
		res += vect[i];
	return res;
} 

int DoubleVector::max()
{
	int i, res = 0;
	double max = vect[0];
	for (i=1; i<size; i++) {
		if (vect[i] > max) {
			res = i;
			max = vect[i];
		}
	}
	return res;
}

void DoubleVector::multiply(double value)
{
	int i;
	for (i=0; i<size; i++)
		vect[i] = vect[i]*value;
	return;
}

void DoubleVector::multiply(int i, double value)
{
	try {
	if (i < size && i >= 0) {
	//gsl_vector_set(vect, i, 
		//		gsl_vector_get(vect, i)*value);
		vect[i] = vect[i]*value;
	} else {
		throw MathlibIndexOutOfRangeException("DoubleVector::multiply(int, double)");
	}
	} catch (MathlibIndexOutOfRangeException & e) {
		e.report();
	}
}

void DoubleVector::multiply(DoubleVector * v)
{
	int i;
	try {
	if (v->size == size) {
	
		for (i=0; i<size; i++) 
			vect[i] = vect[i]*v->get(i);
	} else {
		throw MathlibIncompatibleSizeException("DoubleVector::multiply(DoubleVector *)");
	}
	} catch (MathlibException & e) {
		e.report();
	}
}

void DoubleVector::operator=(double value)
{	
	int i;
	
	for (i=0; i<size; i++)
		vect[i] = value;
	return;
}

void DoubleVector::operator=(DoubleVector & v)
{	
	int i;
	try {
	if (v.size == size) {
		delete [] this->vect;
		this->vect = new double[v.size];
		
		for (i=0; i<size; i++) 
			vect[i] = v.get(i);
	} else {
		throw MathlibIncompatibleSizeException("DoubleVector::=");
	}
	} catch (MathlibException & e) {
		e.report();
	}
}

void DoubleVector::dump() {
	int i;
	
	for (i=0; i<size; i++){
	  printf("%.6f ", this->get(i));
	}
	std::cout << std::endl;
}

}

