/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * GRCRF.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include <fstream>
#include <cstdlib>
#include "GRCRF.h"
#include "TrainerFactory.h"
#include "DecoderFactory.h"
#include "Optimizer.h"
#include "OptimizerLBFGS.h"

namespace models {

	void GRCRF::train() {
		
		training::TrainerFactory tf; 
		
		this->trainer = tf.get_trainer(this->options->training_algorithm, this);
		
		this->trainer->model = this;
		
		this->decoder = NULL;
		
		this->trainer->train_GRCRF();
	}
	
	void GRCRF::decode() {
		
		decoding::DecoderFactory df;

		this->decoder = df.get_decoder(this->options->decoding_algorithm, this);
		
		this->trainer = NULL;
		
		this->decoder->decode();
	}
	
	training::Optimizer * GRCRF::get_optimizer() {
		training::Optimizer * res;
		
		switch (this->options->optimizer) {
			case LBFGS: 
				res = new training::OptimizerLBFGS(this->featureset->num_features, this->options->mhessian, this->options->epsilon);
				break;
				
		} 
	}
	
	void GRCRF::dump()
	{
		/*
		std::filebuf fb;
		fb.open(this->options->model_file.c_str(), std::ios::out);
		std::ostream os(&fb);
		*/
		

		std::ofstream os(this->options->model_file.c_str(), std::ios_base::trunc);
		if (! os.is_open()){
			std::cout << "Error creating/opening model file" << std::endl;
			exit(EXIT_FAILURE);
		}

		this->transducer->dump(&os);
		this->featureset->dump(&os);
		
		os.close();
	}

	void GRCRF::write_predictions()
	{
		this->data->write_predictions();
	}
}

